/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2010, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.test.ws.jaxws.common;

import java.io.StringWriter;
import java.io.Writer;

import org.jboss.logging.Logger;
import org.jboss.wsf.common.DOMUtils;

import javax.xml.soap.SOAPMessage;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

public final class JAXWS_Util
{
   private static final Logger log = Logger.getLogger(JAXWS_Util.class);

   public static DOMResult getSourceAsDOMResult(Source s) throws Exception
   {
      Transformer transformer = TransformerFactory.newInstance().newTransformer();
      transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
      DOMResult result = new DOMResult();
      transformer.transform(s, result);
      return result;
   }

   public static String getDOMResultAsString(DOMResult dr) throws Exception
   {
      DOMSource ds = new DOMSource(dr.getNode());
      Transformer transformer = TransformerFactory.newInstance().newTransformer();
      transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
      transformer.setOutputProperty(OutputKeys.METHOD, "xml");
      Writer out = new StringWriter();
      StreamResult streamResult = new StreamResult();
      streamResult.setWriter(out);
      transformer.transform(ds, streamResult);
      return streamResult.getWriter().toString();
   }

   public static void dumpSOAPMessage(final SOAPMessage msg, final boolean logOnly)
   {
      final StringBuilder sb = new StringBuilder();

      try
      {
         sb.append("--------------------\n");
         sb.append("DUMP OF SOAP MESSAGE\n");
         sb.append("--------------------\n");
         String soapMsgString = DOMUtils.node2String(msg.getSOAPPart());
         sb.append(soapMsgString);
      }
      catch (Exception e)
      {
         log.error(e.getMessage(), e);
         e.printStackTrace();
      }

      if (logOnly)
      {
         log.debug(sb.toString());
      }
      else
      {
         System.err.print(sb.toString());
      }
   }
}
